/* */ 
var __extends = (this && this.__extends) || function(d, b) {
  for (var p in b)
    if (b.hasOwnProperty(p))
      d[p] = b[p];
  function __() {
    this.constructor = d;
  }
  __.prototype = b.prototype;
  d.prototype = new __();
};
var parse5 = require("parse5");
var parser = new parse5.Parser(parse5.TreeAdapters.htmlparser2);
var serializer = new parse5.Serializer(parse5.TreeAdapters.htmlparser2);
var treeAdapter = parser.treeAdapter;
var cssParse = require("css").parse;
var url = require("url");
var collection_1 = require("../facade/collection");
var dom_adapter_1 = require("./dom_adapter");
var lang_1 = require("../facade/lang");
var selector_1 = require("../render/dom/compiler/selector");
var _attrToPropMap = {
  'innerHtml': 'innerHTML',
  'readonly': 'readOnly',
  'tabindex': 'tabIndex'
};
var defDoc = null;
function _notImplemented(methodName) {
  return new lang_1.BaseException('This method is not implemented in Parse5DomAdapter: ' + methodName);
}
var Parse5DomAdapter = (function(_super) {
  __extends(Parse5DomAdapter, _super);
  function Parse5DomAdapter() {
    _super.apply(this, arguments);
  }
  Parse5DomAdapter.makeCurrent = function() {
    dom_adapter_1.setRootDomAdapter(new Parse5DomAdapter());
  };
  Parse5DomAdapter.prototype.logError = function(error) {
    console.error(error);
  };
  Object.defineProperty(Parse5DomAdapter.prototype, "attrToPropMap", {
    get: function() {
      return _attrToPropMap;
    },
    enumerable: true,
    configurable: true
  });
  Parse5DomAdapter.prototype.query = function(selector) {
    throw _notImplemented('query');
  };
  Parse5DomAdapter.prototype.querySelector = function(el, selector) {
    return this.querySelectorAll(el, selector)[0];
  };
  Parse5DomAdapter.prototype.querySelectorAll = function(el, selector) {
    var _this = this;
    var res = collection_1.ListWrapper.create();
    var _recursive = function(result, node, selector, matcher) {
      var cNodes = node.childNodes;
      if (cNodes && cNodes.length > 0) {
        for (var i = 0; i < cNodes.length; i++) {
          var childNode = cNodes[i];
          if (_this.elementMatches(childNode, selector, matcher)) {
            collection_1.ListWrapper.push(result, childNode);
          }
          _recursive(result, childNode, selector, matcher);
        }
      }
    };
    var matcher = new selector_1.SelectorMatcher();
    matcher.addSelectables(selector_1.CssSelector.parse(selector));
    _recursive(res, el, selector, matcher);
    return res;
  };
  Parse5DomAdapter.prototype.elementMatches = function(node, selector, matcher) {
    if (matcher === void 0) {
      matcher = null;
    }
    var result = false;
    if (selector && selector.charAt(0) == "#") {
      result = this.getAttribute(node, 'id') == selector.substring(1);
    } else if (selector) {
      var result = false;
      if (matcher == null) {
        matcher = new selector_1.SelectorMatcher();
        matcher.addSelectables(selector_1.CssSelector.parse(selector));
      }
      var cssSelector = new selector_1.CssSelector();
      cssSelector.setElement(this.tagName(node));
      if (node.attribs) {
        for (var attrName in node.attribs) {
          cssSelector.addAttribute(attrName, node.attribs[attrName]);
        }
      }
      var classList = this.classList(node);
      for (var i = 0; i < classList.length; i++) {
        cssSelector.addClassName(classList[i]);
      }
      matcher.match(cssSelector, function(selector, cb) {
        result = true;
      });
    }
    return result;
  };
  Parse5DomAdapter.prototype.on = function(el, evt, listener) {
    var listenersMap = el._eventListenersMap;
    if (lang_1.isBlank(listenersMap)) {
      var listenersMap = collection_1.StringMapWrapper.create();
      el._eventListenersMap = listenersMap;
    }
    var listeners = collection_1.StringMapWrapper.get(listenersMap, evt);
    if (lang_1.isBlank(listeners)) {
      listeners = collection_1.ListWrapper.create();
    }
    collection_1.ListWrapper.push(listeners, listener);
    collection_1.StringMapWrapper.set(listenersMap, evt, listeners);
  };
  Parse5DomAdapter.prototype.onAndCancel = function(el, evt, listener) {
    this.on(el, evt, listener);
    return function() {
      collection_1.ListWrapper.remove(collection_1.StringMapWrapper.get(el._eventListenersMap, evt), listener);
    };
  };
  Parse5DomAdapter.prototype.dispatchEvent = function(el, evt) {
    if (lang_1.isBlank(evt.target)) {
      evt.target = el;
    }
    if (lang_1.isPresent(el._eventListenersMap)) {
      var listeners = collection_1.StringMapWrapper.get(el._eventListenersMap, evt.type);
      if (lang_1.isPresent(listeners)) {
        for (var i = 0; i < listeners.length; i++) {
          listeners[i](evt);
        }
      }
    }
    if (lang_1.isPresent(el.parent)) {
      this.dispatchEvent(el.parent, evt);
    }
    if (lang_1.isPresent(el._window)) {
      this.dispatchEvent(el._window, evt);
    }
  };
  Parse5DomAdapter.prototype.createMouseEvent = function(eventType) {
    return this.createEvent(eventType);
  };
  Parse5DomAdapter.prototype.createEvent = function(eventType) {
    var evt = {
      type: eventType,
      defaultPrevented: false,
      preventDefault: function() {
        evt.defaultPrevented = true;
      }
    };
    return evt;
  };
  Parse5DomAdapter.prototype.preventDefault = function(evt) {
    evt.returnValue = false;
  };
  Parse5DomAdapter.prototype.getInnerHTML = function(el) {
    return serializer.serialize(this.templateAwareRoot(el));
  };
  Parse5DomAdapter.prototype.getOuterHTML = function(el) {
    serializer.html = '';
    serializer._serializeElement(el);
    return serializer.html;
  };
  Parse5DomAdapter.prototype.nodeName = function(node) {
    return node.tagName;
  };
  Parse5DomAdapter.prototype.nodeValue = function(node) {
    return node.nodeValue;
  };
  Parse5DomAdapter.prototype.type = function(node) {
    throw _notImplemented('type');
  };
  Parse5DomAdapter.prototype.content = function(node) {
    return node.childNodes[0];
  };
  Parse5DomAdapter.prototype.firstChild = function(el) {
    return el.firstChild;
  };
  Parse5DomAdapter.prototype.nextSibling = function(el) {
    return el.nextSibling;
  };
  Parse5DomAdapter.prototype.parentElement = function(el) {
    return el.parent;
  };
  Parse5DomAdapter.prototype.childNodes = function(el) {
    return el.childNodes;
  };
  Parse5DomAdapter.prototype.childNodesAsList = function(el) {
    var childNodes = el.childNodes;
    var res = collection_1.ListWrapper.createFixedSize(childNodes.length);
    for (var i = 0; i < childNodes.length; i++) {
      res[i] = childNodes[i];
    }
    return res;
  };
  Parse5DomAdapter.prototype.clearNodes = function(el) {
    while (el.childNodes.length > 0) {
      this.remove(el.childNodes[0]);
    }
  };
  Parse5DomAdapter.prototype.appendChild = function(el, node) {
    this.remove(node);
    treeAdapter.appendChild(this.templateAwareRoot(el), node);
  };
  Parse5DomAdapter.prototype.removeChild = function(el, node) {
    if (collection_1.ListWrapper.contains(el.childNodes, node)) {
      this.remove(node);
    }
  };
  Parse5DomAdapter.prototype.remove = function(el) {
    var parent = el.parent;
    if (parent) {
      var index = parent.childNodes.indexOf(el);
      parent.childNodes.splice(index, 1);
    }
    var prev = el.previousSibling;
    var next = el.nextSibling;
    if (prev) {
      prev.next = next;
    }
    if (next) {
      next.prev = prev;
    }
    el.prev = null;
    el.next = null;
    el.parent = null;
    return el;
  };
  Parse5DomAdapter.prototype.insertBefore = function(el, node) {
    this.remove(node);
    treeAdapter.insertBefore(el.parent, node, el);
  };
  Parse5DomAdapter.prototype.insertAllBefore = function(el, nodes) {
    var _this = this;
    collection_1.ListWrapper.forEach(nodes, function(n) {
      _this.insertBefore(el, n);
    });
  };
  Parse5DomAdapter.prototype.insertAfter = function(el, node) {
    if (el.nextSibling) {
      this.insertBefore(el.nextSibling, node);
    } else {
      this.appendChild(el.parent, node);
    }
  };
  Parse5DomAdapter.prototype.setInnerHTML = function(el, value) {
    this.clearNodes(el);
    var content = parser.parseFragment(value);
    for (var i = 0; i < content.childNodes.length; i++) {
      treeAdapter.appendChild(el, content.childNodes[i]);
    }
  };
  Parse5DomAdapter.prototype.getText = function(el) {
    if (this.isTextNode(el)) {
      return el.data;
    } else if (el.childNodes.length == 0) {
      return "";
    } else {
      var textContent = "";
      for (var i = 0; i < el.childNodes.length; i++) {
        textContent += this.getText(el.childNodes[i]);
      }
      return textContent;
    }
  };
  Parse5DomAdapter.prototype.setText = function(el, value) {
    if (this.isTextNode(el)) {
      el.data = value;
    } else {
      this.clearNodes(el);
      treeAdapter.insertText(el, value);
    }
  };
  Parse5DomAdapter.prototype.getValue = function(el) {
    return el.value;
  };
  Parse5DomAdapter.prototype.setValue = function(el, value) {
    el.value = value;
  };
  Parse5DomAdapter.prototype.getChecked = function(el) {
    return el.checked;
  };
  Parse5DomAdapter.prototype.setChecked = function(el, value) {
    el.checked = value;
  };
  Parse5DomAdapter.prototype.createTemplate = function(html) {
    var template = treeAdapter.createElement("template", 'http://www.w3.org/1999/xhtml', []);
    var content = parser.parseFragment(html);
    treeAdapter.appendChild(template, content);
    return template;
  };
  Parse5DomAdapter.prototype.createElement = function(tagName) {
    return treeAdapter.createElement(tagName, 'http://www.w3.org/1999/xhtml', []);
  };
  Parse5DomAdapter.prototype.createTextNode = function(text) {
    throw _notImplemented('createTextNode');
  };
  Parse5DomAdapter.prototype.createScriptTag = function(attrName, attrValue) {
    return treeAdapter.createElement("script", 'http://www.w3.org/1999/xhtml', [{
      name: attrName,
      value: attrValue
    }]);
  };
  Parse5DomAdapter.prototype.createStyleElement = function(css) {
    var style = this.createElement('style');
    this.setText(style, css);
    return style;
  };
  Parse5DomAdapter.prototype.createShadowRoot = function(el) {
    el.shadowRoot = treeAdapter.createDocumentFragment();
    el.shadowRoot.parent = el;
    return el.shadowRoot;
  };
  Parse5DomAdapter.prototype.getShadowRoot = function(el) {
    return el.shadowRoot;
  };
  Parse5DomAdapter.prototype.getHost = function(el) {
    return el.host;
  };
  Parse5DomAdapter.prototype.getDistributedNodes = function(el) {
    throw _notImplemented('getDistributedNodes');
  };
  Parse5DomAdapter.prototype.clone = function(node) {
    if (node.type === 'root') {
      var serialized = serializer.serialize(node);
      var newParser = new parse5.Parser(parse5.TreeAdapters.htmlparser2);
      return newParser.parseFragment(serialized);
    } else {
      var temp = treeAdapter.createElement("template", null, []);
      treeAdapter.appendChild(temp, node);
      var serialized = serializer.serialize(temp);
      var newParser = new parse5.Parser(parse5.TreeAdapters.htmlparser2);
      return newParser.parseFragment(serialized).childNodes[0];
    }
  };
  Parse5DomAdapter.prototype.hasProperty = function(element, name) {
    return _HTMLElementPropertyList.indexOf(name) > -1;
  };
  Parse5DomAdapter.prototype.getElementsByClassName = function(element, name) {
    return this.querySelectorAll(element, "." + name);
  };
  Parse5DomAdapter.prototype.getElementsByTagName = function(element, name) {
    throw _notImplemented('getElementsByTagName');
  };
  Parse5DomAdapter.prototype.classList = function(element) {
    var classAttrValue = null;
    var attributes = element.attribs;
    if (attributes && attributes.hasOwnProperty("class")) {
      classAttrValue = attributes["class"];
    }
    return classAttrValue ? classAttrValue.trim().split(/\s+/g) : [];
  };
  Parse5DomAdapter.prototype.addClass = function(element, classname) {
    var classList = this.classList(element);
    var index = classList.indexOf(classname);
    if (index == -1) {
      collection_1.ListWrapper.push(classList, classname);
      element.attribs["class"] = element.className = collection_1.ListWrapper.join(classList, " ");
    }
  };
  Parse5DomAdapter.prototype.removeClass = function(element, classname) {
    var classList = this.classList(element);
    var index = classList.indexOf(classname);
    if (index > -1) {
      classList.splice(index, 1);
      element.attribs["class"] = element.className = collection_1.ListWrapper.join(classList, " ");
    }
  };
  Parse5DomAdapter.prototype.hasClass = function(element, classname) {
    return collection_1.ListWrapper.contains(this.classList(element), classname);
  };
  Parse5DomAdapter.prototype._readStyleAttribute = function(element) {
    var styleMap = {};
    var attributes = element.attribs;
    if (attributes && attributes.hasOwnProperty("style")) {
      var styleAttrValue = attributes["style"];
      var styleList = styleAttrValue.split(/;+/g);
      for (var i = 0; i < styleList.length; i++) {
        if (styleList[i].length > 0) {
          var elems = styleList[i].split(/:+/g);
          styleMap[elems[0].trim()] = elems[1].trim();
        }
      }
    }
    return styleMap;
  };
  Parse5DomAdapter.prototype._writeStyleAttribute = function(element, styleMap) {
    var styleAttrValue = "";
    for (var key in styleMap) {
      var newValue = styleMap[key];
      if (newValue && newValue.length > 0) {
        styleAttrValue += key + ":" + styleMap[key] + ";";
      }
    }
    element.attribs["style"] = styleAttrValue;
  };
  Parse5DomAdapter.prototype.setStyle = function(element, stylename, stylevalue) {
    var styleMap = this._readStyleAttribute(element);
    styleMap[stylename] = stylevalue;
    this._writeStyleAttribute(element, styleMap);
  };
  Parse5DomAdapter.prototype.removeStyle = function(element, stylename) {
    this.setStyle(element, stylename, null);
  };
  Parse5DomAdapter.prototype.getStyle = function(element, stylename) {
    var styleMap = this._readStyleAttribute(element);
    return styleMap.hasOwnProperty(stylename) ? styleMap[stylename] : "";
  };
  Parse5DomAdapter.prototype.tagName = function(element) {
    return element.tagName == "style" ? "STYLE" : element.tagName;
  };
  Parse5DomAdapter.prototype.attributeMap = function(element) {
    var res = collection_1.MapWrapper.create();
    var elAttrs = treeAdapter.getAttrList(element);
    for (var i = 0; i < elAttrs.length; i++) {
      var attrib = elAttrs[i];
      collection_1.MapWrapper.set(res, attrib.name, attrib.value);
    }
    return res;
  };
  Parse5DomAdapter.prototype.hasAttribute = function(element, attribute) {
    return element.attribs && element.attribs.hasOwnProperty(attribute);
  };
  Parse5DomAdapter.prototype.getAttribute = function(element, attribute) {
    return element.attribs && element.attribs.hasOwnProperty(attribute) ? element.attribs[attribute] : null;
  };
  Parse5DomAdapter.prototype.setAttribute = function(element, attribute, value) {
    if (attribute) {
      element.attribs[attribute] = value;
    }
  };
  Parse5DomAdapter.prototype.removeAttribute = function(element, attribute) {
    if (attribute) {
      collection_1.StringMapWrapper.delete(element.attribs, attribute);
    }
  };
  Parse5DomAdapter.prototype.templateAwareRoot = function(el) {
    return this.isTemplateElement(el) ? this.content(el) : el;
  };
  Parse5DomAdapter.prototype.createHtmlDocument = function() {
    var newDoc = treeAdapter.createDocument();
    newDoc.title = "fake title";
    var head = treeAdapter.createElement("head", null, []);
    var body = treeAdapter.createElement("body", 'http://www.w3.org/1999/xhtml', []);
    this.appendChild(newDoc, head);
    this.appendChild(newDoc, body);
    collection_1.StringMapWrapper.set(newDoc, "head", head);
    collection_1.StringMapWrapper.set(newDoc, "body", body);
    collection_1.StringMapWrapper.set(newDoc, "_window", collection_1.StringMapWrapper.create());
    return newDoc;
  };
  Parse5DomAdapter.prototype.defaultDoc = function() {
    if (defDoc === null) {
      defDoc = this.createHtmlDocument();
    }
    return defDoc;
  };
  Parse5DomAdapter.prototype.getBoundingClientRect = function(el) {
    return {
      left: 0,
      top: 0,
      width: 0,
      height: 0
    };
  };
  Parse5DomAdapter.prototype.getTitle = function() {
    return this.defaultDoc().title || "";
  };
  Parse5DomAdapter.prototype.setTitle = function(newTitle) {
    this.defaultDoc().title = newTitle;
  };
  Parse5DomAdapter.prototype.isTemplateElement = function(el) {
    return this.isElementNode(el) && this.tagName(el) === "template";
  };
  Parse5DomAdapter.prototype.isTextNode = function(node) {
    return treeAdapter.isTextNode(node);
  };
  Parse5DomAdapter.prototype.isCommentNode = function(node) {
    return treeAdapter.isCommentNode(node);
  };
  Parse5DomAdapter.prototype.isElementNode = function(node) {
    return node ? treeAdapter.isElementNode(node) : false;
  };
  Parse5DomAdapter.prototype.hasShadowRoot = function(node) {
    return lang_1.isPresent(node.shadowRoot);
  };
  Parse5DomAdapter.prototype.isShadowRoot = function(node) {
    return this.getShadowRoot(node) == node;
  };
  Parse5DomAdapter.prototype.importIntoDoc = function(node) {
    return this.clone(node);
  };
  Parse5DomAdapter.prototype.isPageRule = function(rule) {
    return rule.type === 6;
  };
  Parse5DomAdapter.prototype.isStyleRule = function(rule) {
    return rule.type === 1;
  };
  Parse5DomAdapter.prototype.isMediaRule = function(rule) {
    return rule.type === 4;
  };
  Parse5DomAdapter.prototype.isKeyframesRule = function(rule) {
    return rule.type === 7;
  };
  Parse5DomAdapter.prototype.getHref = function(el) {
    return el.href;
  };
  Parse5DomAdapter.prototype.resolveAndSetHref = function(el, baseUrl, href) {
    if (href == null) {
      el.href = baseUrl;
    } else {
      el.href = url.resolve(baseUrl, href);
    }
  };
  Parse5DomAdapter.prototype._buildRules = function(parsedRules, css) {
    var rules = collection_1.ListWrapper.create();
    for (var i = 0; i < parsedRules.length; i++) {
      var parsedRule = parsedRules[i];
      var rule = collection_1.StringMapWrapper.create();
      collection_1.StringMapWrapper.set(rule, "cssText", css);
      collection_1.StringMapWrapper.set(rule, "style", {
        content: "",
        cssText: ""
      });
      if (parsedRule.type == "rule") {
        collection_1.StringMapWrapper.set(rule, "type", 1);
        collection_1.StringMapWrapper.set(rule, "selectorText", parsedRule.selectors.join(", ").replace(/\s{2,}/g, " ").replace(/\s*~\s*/g, " ~ ").replace(/\s*\+\s*/g, " + ").replace(/\s*>\s*/g, " > ").replace(/\[(\w+)=(\w+)\]/g, '[$1="$2"]'));
        if (lang_1.isBlank(parsedRule.declarations)) {
          continue;
        }
        for (var j = 0; j < parsedRule.declarations.length; j++) {
          var declaration = parsedRule.declarations[j];
          collection_1.StringMapWrapper.set(collection_1.StringMapWrapper.get(rule, "style"), declaration.property, declaration.value);
          collection_1.StringMapWrapper.get(rule, "style").cssText += declaration.property + ": " + declaration.value + ";";
        }
      } else if (parsedRule.type == "media") {
        collection_1.StringMapWrapper.set(rule, "type", 4);
        collection_1.StringMapWrapper.set(rule, "media", {mediaText: parsedRule.media});
        if (parsedRule.rules) {
          collection_1.StringMapWrapper.set(rule, "cssRules", this._buildRules(parsedRule.rules));
        }
      }
      collection_1.ListWrapper.push(rules, rule);
    }
    return rules;
  };
  Parse5DomAdapter.prototype.cssToRules = function(css) {
    css = css.replace(/url\(\'(.+)\'\)/g, 'url($1)');
    var rules = collection_1.ListWrapper.create();
    var parsedCSS = cssParse(css, {silent: true});
    if (parsedCSS.stylesheet && parsedCSS.stylesheet.rules) {
      rules = this._buildRules(parsedCSS.stylesheet.rules, css);
    }
    return rules;
  };
  Parse5DomAdapter.prototype.supportsDOMEvents = function() {
    return false;
  };
  Parse5DomAdapter.prototype.supportsNativeShadowDOM = function() {
    return false;
  };
  Parse5DomAdapter.prototype.getGlobalEventTarget = function(target) {
    if (target == "window") {
      return this.defaultDoc()._window;
    } else if (target == "document") {
      return this.defaultDoc();
    } else if (target == "body") {
      return this.defaultDoc().body;
    }
  };
  Parse5DomAdapter.prototype.getHistory = function() {
    throw 'not implemented';
  };
  Parse5DomAdapter.prototype.getLocation = function() {
    throw 'not implemented';
  };
  Parse5DomAdapter.prototype.getUserAgent = function() {
    return "Fake user agent";
  };
  Parse5DomAdapter.prototype.getData = function(el, name) {
    return this.getAttribute(el, 'data-' + name);
  };
  Parse5DomAdapter.prototype.setData = function(el, name, value) {
    this.setAttribute(el, 'data-' + name, value);
  };
  Parse5DomAdapter.prototype.setGlobalVar = function(name, value) {
    lang_1.global[name] = value;
  };
  return Parse5DomAdapter;
})(dom_adapter_1.DomAdapter);
exports.Parse5DomAdapter = Parse5DomAdapter;
var _HTMLElementPropertyList = ["webkitEntries", "incremental", "webkitdirectory", "selectionDirection", "selectionEnd", "selectionStart", "labels", "validationMessage", "validity", "willValidate", "width", "valueAsNumber", "valueAsDate", "value", "useMap", "defaultValue", "type", "step", "src", "size", "required", "readOnly", "placeholder", "pattern", "name", "multiple", "min", "minLength", "maxLength", "max", "list", "indeterminate", "height", "formTarget", "formNoValidate", "formMethod", "formEnctype", "formAction", "files", "form", "disabled", "dirName", "checked", "defaultChecked", "autofocus", "autocomplete", "alt", "align", "accept", "onautocompleteerror", "onautocomplete", "onwaiting", "onvolumechange", "ontoggle", "ontimeupdate", "onsuspend", "onsubmit", "onstalled", "onshow", "onselect", "onseeking", "onseeked", "onscroll", "onresize", "onreset", "onratechange", "onprogress", "onplaying", "onplay", "onpause", "onmousewheel", "onmouseup", "onmouseover", "onmouseout", "onmousemove", "onmouseleave", "onmouseenter", "onmousedown", "onloadstart", "onloadedmetadata", "onloadeddata", "onload", "onkeyup", "onkeypress", "onkeydown", "oninvalid", "oninput", "onfocus", "onerror", "onended", "onemptied", "ondurationchange", "ondrop", "ondragstart", "ondragover", "ondragleave", "ondragenter", "ondragend", "ondrag", "ondblclick", "oncuechange", "oncontextmenu", "onclose", "onclick", "onchange", "oncanplaythrough", "oncanplay", "oncancel", "onblur", "onabort", "spellcheck", "isContentEditable", "contentEditable", "outerText", "innerText", "accessKey", "hidden", "webkitdropzone", "draggable", "tabIndex", "dir", "translate", "lang", "title", "childElementCount", "lastElementChild", "firstElementChild", "children", "onwebkitfullscreenerror", "onwebkitfullscreenchange", "nextElementSibling", "previousElementSibling", "onwheel", "onselectstart", "onsearch", "onpaste", "oncut", "oncopy", "onbeforepaste", "onbeforecut", "onbeforecopy", "shadowRoot", "dataset", "classList", "className", "outerHTML", "innerHTML", "scrollHeight", "scrollWidth", "scrollTop", "scrollLeft", "clientHeight", "clientWidth", "clientTop", "clientLeft", "offsetParent", "offsetHeight", "offsetWidth", "offsetTop", "offsetLeft", "localName", "prefix", "namespaceURI", "id", "style", "attributes", "tagName", "parentElement", "textContent", "baseURI", "ownerDocument", "nextSibling", "previousSibling", "lastChild", "firstChild", "childNodes", "parentNode", "nodeType", "nodeValue", "nodeName", "closure_lm_714617", "__jsaction"];
exports.__esModule = true;
