/* */ 
(function(Buffer) {
  'use strict';
  var sign = require("./sign");
  var verify = require("./verify");
  var stream = require("stream");
  var inherits = require("inherits");
  var _algos = require("./algos");
  var createHash = require("create-hash");
  var algos = {};
  Object.keys(_algos).forEach(function(key) {
    algos[key] = algos[key.toLowerCase()] = _algos[key];
  });
  exports.createSign = exports.Sign = createSign;
  function createSign(algorithm) {
    return new Sign(algorithm);
  }
  exports.createVerify = exports.Verify = createVerify;
  function createVerify(algorithm) {
    return new Verify(algorithm);
  }
  inherits(Sign, stream.Writable);
  function Sign(algorithm) {
    stream.Writable.call(this);
    var data = algos[algorithm];
    if (!data)
      throw new Error('Unknown message digest');
    this._hashType = data.hash;
    this._hash = createHash(data.hash);
    this._tag = data.id;
    this._signType = data.sign;
  }
  Sign.prototype._write = function _write(data, _, done) {
    this._hash.update(data);
    done();
  };
  Sign.prototype.update = function update(data, enc) {
    if (typeof data === 'string')
      data = new Buffer(data, enc);
    this._hash.update(data);
    return this;
  };
  Sign.prototype.sign = function signMethod(key, enc) {
    this.end();
    var hash = this._hash.digest();
    var sig = sign(Buffer.concat([this._tag, hash]), key, this._hashType, this._signType);
    if (enc) {
      sig = sig.toString(enc);
    }
    return sig;
  };
  inherits(Verify, stream.Writable);
  function Verify(algorithm) {
    stream.Writable.call(this);
    var data = algos[algorithm];
    if (!data)
      throw new Error('Unknown message digest');
    this._hash = createHash(data.hash);
    this._tag = data.id;
    this._signType = data.sign;
  }
  Verify.prototype._write = function _write(data, _, done) {
    this._hash.update(data);
    done();
  };
  Verify.prototype.update = function update(data, enc) {
    if (typeof data === 'string')
      data = new Buffer(data, enc);
    this._hash.update(data);
    return this;
  };
  Verify.prototype.verify = function verifyMethod(key, sig, enc) {
    this.end();
    var hash = this._hash.digest();
    if (typeof sig === 'string')
      sig = new Buffer(sig, enc);
    return verify(sig, Buffer.concat([this._tag, hash]), key, this._signType);
  };
})(require("buffer").Buffer);
